/*
 *	Scoutzknivez Fun
 *	Version: 0.3
 *	by v3x - slayerized@gmail.com
 *
 *	Cvars:
 *	 - scoutz_nades "abc"
 *	   a: HE grenades
 *	   b: Flashbang
 *	   c: Smoke grenade
 *
 *	 - scoutz_awp <0|1>
 *	   0: Do not spawn AWP at top on random rounds
 *	   1: Spawn AWP at top on random rounds
 *
 *	What it does:
 *	 - Random low gravity on each new round
 *	 - Random knife rounds
 *	 - Random knife rounds with noclip
 *	 - Random AWP spawn on top of map
 *	 - On knife rounds each player will glow a random color
 *	 - Nades given depending on cvar value
 *	 - Gives an extra 10 bullets for scout
 *
 *	About AWP rounds:
 *	 If random AWP rounds are activated, a glowing AWP will be spawned at the top of
 *	 the map ( the highest ledge ). Whoever gets to it first gets 145hp and can keep
 *	 the AWP for that round only. The trick is, the player cannot drop the AWP once 
 *	 he has it. He will also be forced to drop all grenades. 
 *
 *	Required Modules: Engine, Fun
 *
 *	Notes:
 *	 - This will only work on the map 'scoutzknivez'
 *	 - If you don't normally use 800 as your server's gravity, go to the line that 
 *	   says DEFAULT_GRAV and edit the value.
 */

#include <amxmodx>
#include <engine>
#include <fun>

#define PLUGIN		"Scoutzknivez Fun"
#define VERSION		"0.3"
#define AUTHOR		"v3x"

public plugin_init()
{
  register_plugin(PLUGIN, VERSION, AUTHOR);
  register_event("CurWeapon", "Check_Knife", "be", "2!29");
  register_event("ResetHUD", "Player_Spawn", "b");
  register_logevent("New_Round", 2, "0=World triggered", "1=Round_Start");
  register_cvar("scoutz_nades", "abc");
  register_cvar("scoutz_awp", "1");
  register_clcmd("fullupdate", "Block_Cmd");
  register_clcmd("drop", "Handle_Drop");
}

#define	NADE_HE		1
#define	NADE_FB		2
#define NADE_SM		4

#define MAX_COLORS	9

#define DEFAULT_GRAV	800

new g_iGravity[3] = { 250, 200, 100 };

new bool:g_bScoutz = false;
new bool:g_bKnifeRound = false;
new bool:g_bNoClipRound = false;
new bool:g_bHasAwp[33] = { false, ... };

new g_iColors[MAX_COLORS][3] = 
{
  {255,255,255},{255,0,0},
  {0,0,255},{0,255,0},
  {255,255,0},{255,0,255},
  {0,255,255},{227,96,8},
  {45,89,116}
}

public plugin_cfg()
{
  new szMapName[64];
  get_mapname(szMapName, 63);

  if(equali(szMapName, "scoutzknivez"))
  {
    g_bScoutz = true;
    server_print("*** Scoutknivez fun loaded ***");
  }
}

public client_connect( id )
  g_bHasAwp[id] = false;

public Check_Knife( id )
{
  if(!g_bScoutz || !g_bKnifeRound)
    return PLUGIN_CONTINUE;

  if(g_bKnifeRound)
  {
    new iClip, iAmmo, iWeap = get_user_weapon(id, iClip, iAmmo);

    if(iWeap != 4 && iWeap != 9 && iWeap != 25 && iWeap != 29)
      engclient_cmd(id, "weapon_knife");
  }
  return PLUGIN_CONTINUE;
}

public New_Round()
{
  if(!g_bScoutz)
    return PLUGIN_CONTINUE;

  server_cmd("sv_gravity %i", g_iGravity[random_num(0,2)]);

  new ent;
  while((ent = find_ent_by_class(ent, "smf_awp")) != 0)
    remove_entity(ent);

  if(random_num(0, 100) <= 25)
  {
    client_print(0, print_center, "*** KNIFE ROUND ***");
    g_bKnifeRound = true;

    if(random_num(0, 100) <= 25)
    {
      client_print(0, print_center, "*** KNIFE/NOCLIP ROUND ***");
      g_bNoClipRound = true;
    }
    else
      g_bNoClipRound = false;
  }
  else
  {
    g_bKnifeRound = false;
    g_bNoClipRound = false;

    if(get_cvar_num("scoutz_awp"))
    {
      if(random_num(0, 100) <= 45)
      {
        client_print(0, print_chat, "*** Get to the top of the map and grab the AWP and get a health bonus as well! ***");
        Spawn_Awp();
      }
    }
  }

  new iPlayers[32], iNum, i;
  get_players(iPlayers, iNum, "a");

  for(i = 0; i <= iNum; i++)
  {
    g_bHasAwp[iPlayers[i]] = false;
 
    if(g_bKnifeRound)
      engclient_cmd(iPlayers[i], "weapon_knife");
  }

  return PLUGIN_CONTINUE;
}

public Player_Spawn( id )
{
  if(!g_bScoutz)
    return;

  if(g_bKnifeRound)
  {
    Set_Glow( id );

    if(g_bNoClipRound)
      set_user_noclip(id, 1);
    else
      set_user_noclip(id, 0);
  }
  else
  {
    if(!g_bNoClipRound)
      set_user_noclip(id, 0);

    set_user_rendering(id, kRenderFxNone, 255, 255, 255, kRenderNormal, 255)
  }
  set_task(0.1, "Give_Stuff", id);
  set_task(0.2, "Scouty", id);
}

public Give_Stuff( id )
{
  give_item(id, "item_assaultsuit");
  give_item(id, "weapon_scout");

  if(get_nade_flags() & NADE_HE)
    give_item(id, "weapon_hegrenade");
  if(get_nade_flags() & NADE_FB)
    give_item(id, "weapon_flashbang");
  if(get_nade_flags() & NADE_SM)
    give_item(id, "weapon_hegrenade");
}

public Scouty( id )
{
  engclient_cmd(id, "weapon_scout");

  if(g_bKnifeRound)
    engclient_cmd(id, "weapon_knife");
}

public Set_Glow( id )
{
  new iRandColor = random_num(0, MAX_COLORS-1);

  new iRed =   g_iColors[iRandColor][0];
  new iGreen = g_iColors[iRandColor][1];
  new iBlue =  g_iColors[iRandColor][2];

  set_user_rendering(id, kRenderFxGlowShell, iRed, iGreen, iBlue, kRenderNormal, 25)
}

public Block_Cmd( id )
  return PLUGIN_HANDLED;

public Handle_Drop( id )
{
  if(!g_bScoutz)
    return PLUGIN_CONTINUE;

  if(g_bHasAwp[id])
  {
    client_print(id, print_center, "You have the AWP, you cannot drop it!");
    return PLUGIN_HANDLED;
  }
  return PLUGIN_CONTINUE;
}

public get_nade_flags()
{
  new szFlags[33];
  get_cvar_string("scoutz_nades", szFlags, 32);

  return read_flags(szFlags);
}

public Spawn_Awp()
{
  new iAwp = create_entity("info_target");
  if(iAwp == 0) 
    return PLUGIN_HANDLED;

  entity_set_string(iAwp, EV_SZ_classname, "smf_awp");
  entity_set_model(iAwp, "models/w_awp.mdl");

  entity_set_size(iAwp, Float:{-2.0,-2.0,-1.0}, Float:{2.0,2.0,1.0});

  entity_set_int(iAwp, EV_INT_movetype, MOVETYPE_TOSS);
  entity_set_int(iAwp, EV_INT_solid, SOLID_BBOX);

  new iRed =   random_num(15, 255); 
  new iGreen = random_num(15, 255);
  new iBlue =  random_num(15, 255);

  set_rendering(iAwp, kRenderFxGlowShell, iRed, iGreen, iBlue, kRenderNormal, 75);

  new Float:fOrigin[3];

  fOrigin[0] = -389.0;
  fOrigin[1] = 123.0;
  fOrigin[2] = 460.0;

  entity_set_origin(iAwp, fOrigin);

  drop_to_floor(iAwp);

  return PLUGIN_HANDLED;
}

public pfn_touch(ptr, ptd)
{
  if(!g_bScoutz)
    return;

  if(ptr > 0 && ptd > 0 && ptd <= get_maxplayers())
  {
    if(!is_valid_ent(ptr) || !is_valid_ent(ptd))
      return;

    new pToucher[33], pTouched[33];
    entity_get_string(ptr, EV_SZ_classname, pToucher, 32);
    entity_get_string(ptd, EV_SZ_classname, pTouched, 32);

    if(equali(pToucher, "smf_awp") && equali(pTouched, "player"))
    {
      if(!is_user_connected(ptd) || !is_user_alive(ptd))
        return;

      new szUserName[33];
      get_user_name(ptd, szUserName, 32);

      client_print(0, print_center, "%s has picked up the AWP", szUserName);

      set_user_health(ptd, 145);
      strip_user_weapons(ptd);

      set_task(0.1, "Give_Items", ptd);

      g_bHasAwp[ptd] = true;

      remove_entity(ptr);

      return;
    }
  }
}

public Give_Items( id )
{
  give_item(id, "weapon_knife");
  give_item(id, "weapon_awp");
  give_item(id, "weapon_awp");
  give_item(id, "weapon_awp");
}

public plugin_precache()
  precache_model("models/w_awp.mdl");

public plugin_end()
{
  if(g_bScoutz)
    server_cmd("sv_gravity %i", DEFAULT_GRAV);
}